@@ -1,11 +1,8 @@
-#!/usr/bin/python2.7
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
+# Copyright 2012-2021 Univention GmbH
 #
-# Univention Common Python Library
-#
-# Copyright 2012-2017 Univention GmbH
-#
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -28,24 +25,29 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
 
-'''This module abstracts the handling of at-jobs, each job is encapsulated by
-the class AtJob. Use the method add() in order to add a new command to the
-queue of at-jobs. Use the methods list() and load() to get a list of all
+"""
+Univention common Python library for handling :program:`at` jobs.
+
+This module abstracts the handling of at-jobs, each job is encapsulated by
+the class :py:class:`AtJob`. Use the method :py:meth:`add` in order to add a new command to the
+queue of at-jobs. Use the methods :py:meth:`list` and :py:meth:`load` to get a list of all
 registered jobs or to load a specific job given an ID, respectively. The module
-uses time stamps in seconds for scheduling jobs.'''
+uses time stamps in seconds for scheduling jobs.
+"""
 
 import datetime
 import subprocess
 import re
 import locale
+from typing import Dict, List, Mapping, Optional, Union  # noqa F401
 
 __all__ = ['add', 'list', 'load', 'remove', 'reschedule', 'AtJob']
 
 # internal formatting strings and regexps
 _regWhiteSpace = re.compile(r'\s+')
-_regJobNr = re.compile(r'job\s+(\d+)')
+_regJobNr = re.compile(r'job\s+(\d+)'.encode('ASCII'))
 _dateTimeFormatRead = '%a %b %d %H:%M:%S %Y'
 _dateTimeFormatWrite = '%Y-%m-%d %H:%M'
 _timeFormatWrite = '%H:%M'
@@ -56,13 +58,20 @@ COMMENT_PREFIX = '# Comment: '
 
 
 def add(cmd, execTime=None, comments={}):
-	'''Add a new command to the job queue given a time (in seconds since
-	the epoch or as a datetime object) at which the job will be
-	executed. Optionally comments can be provided as a dictionary. The
-	data is assoziated with the job.'''
+	# type: (str, Union[None, int, float, datetime.datetime], Optional[Mapping[str, str]]) -> Optional[AtJob]
+	"""
+	Add a new command to the job queue given a time
+	at which the job will be executed.
+
+	:param execTime: execution time either as seconds since the epoch or as a :py:class:`datetime.datetime` instance. Defaults to `now`.
+	:type execTime: int or float or datetime.datetime or None
+	:param dict comments: A optional dictionary with comments to be associated with the job.
+	:returns: The created job or `None`.
+	:rtype: AtJob or None
+	"""
 
 	if isinstance(execTime, (int, float)):
-		start = datetime.datetime.fromtimestamp(execTime)
+		start = datetime.datetime.fromtimestamp(execTime)  # type: Optional[datetime.datetime]
 	else:
 		start = execTime
 
@@ -83,27 +92,37 @@ def add(cmd, execTime=None, comments={}):
 				value = value.decode('utf-8')
 			except UnicodeDecodeError:
 				value = value.decode('latin-1')
-		return (u'%s' % (value,)).encode('unicode_escape')
+		return (u'%s' % (value,)).encode('unicode_escape').decode('ASCII')
 
 	# add comments
 	if comments:
-		cmd = '\n'.join(map(lambda c: '%s%s:%s' % (COMMENT_PREFIX, _encode_comment(c[0]).replace(':', ''), _encode_comment(c[1])), comments.items())) + '\n' + SCRIPT_PREFIX + '\n' + cmd
+		cmd = '\n'.join('%s%s:%s' % (COMMENT_PREFIX, _encode_comment(c[0]).replace(':', ''), _encode_comment(c[1])) for c in comments.items()) + '\n' + SCRIPT_PREFIX + '\n' + cmd
 
 	# add job
 	p = subprocess.Popen(atCmd, stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
 
 	# send the job to stdin
-	out = p.communicate(cmd)
+	out = p.communicate(cmd.encode('UTF-8'))
 
 	# parse output and return job
-	matches = _regJobNr.findall('\n'.join(out))
+	matches = _regJobNr.findall(b'\n'.join(out))
 	if matches:
 		return load(int(matches[0]))
 	return None
 
 
 def reschedule(nr, execTime=None):
-	"""Re-schedules the at job with the given number for the specified time"""
+	# type: (int, Optional[float]) -> Optional[AtJob]
+	"""
+	Re-schedules the at job with the given number for the specified time.
+
+	:param int nr: The job number.
+	:param execTime: execution time either as seconds since the epoch or as a :py:class:`datetime.datetime` instance. Defaults to `now`.
+	:type execTime: int or float or datetime.datetime or None
+	:returns: The created job or `None`.
+	:rtype: AtJob or None
+	:raises: AttributeError: if the job cannot be found.
+	"""
 	atjob = load(nr, extended=True)
 	if atjob is None:
 		raise AttributeError('Could not find at job %s' % nr)
@@ -115,16 +134,23 @@ def reschedule(nr, execTime=None):
 
 
 def list(extended=False):
-	'''Returns a list of all registered jobs as instances of AtJob. If
-	extended is set to True the parser also extras the comments and the
-	command to execute. This can be used to re-schedule a job.'''
+	# type: (bool) -> List[AtJob]
+	"""
+	Returns a list of all registered jobs.
+
+	:param bool extended: If set to `True` also the comments and the command to execute are fetched.
+	:returns: A list of :py:class:`AtJob` instances.
+	:rtype: list[AtJob]
+
+	This can be used to re-schedule a job.
+	"""
 	p = subprocess.Popen('/usr/bin/atq', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-	out = p.communicate()[0].splitlines()
+	out = p.communicate()[0].decode('UTF-8', 'replace').splitlines()
 	jobs = []
 	for line in out:
 		ijob = _parseJob(line)
 		if ijob:
-			jobs.append(_parseJob(line))
+			jobs.append(ijob)
 
 	if extended:
 		for job in jobs:
@@ -134,8 +160,15 @@ def list(extended=False):
 
 
 def load(nr, extended=False):
-	'''Load a job given a particular ID. Returns None if job does not exist,
-	otherwise an instance of AtJob is returned.'''
+	# type: (int, bool) -> Optional[AtJob]
+	"""
+	Load the job given.
+
+	:param nr: Job number.
+	:param bool extended: If set to `True` also the comments and the command to execute are fetched.
+	:returns: `None` if job does not exist, otherwise an instance of :py:class:`AtJob`.
+	:rtype: AtJob
+	"""
 	result = [p for p in list(extended) if p.nr == nr]
 	if len(result):
 		return result[0]
@@ -143,15 +176,28 @@ def load(nr, extended=False):
 
 
 def remove(nr):
-	"""Removes the at job with the given number"""
-	result = [p for p in list() if p.nr == nr]
-	if len(result):
-		return result[0].rm()
+	# type: (int) -> Optional[int]
+	"""
+	Removes the at job with the given number.
+
+	:param int nr: Job number.
+	"""
+	for job in list():
+		if job.nr == nr:
+			return job.rm()
+	return None
 
 
 def _parseScript(job):
+	# type: (AtJob) -> None
+	"""
+	Internal function to load the job details by parsing the job of :command:`atq`.
+
+	:param AtJob job: A job.
+	"""
+	# FIXME: This should be a method of the class.
 	p = subprocess.Popen(['/usr/bin/at', '-c', str(job.nr)], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-	out = p.communicate()[0].splitlines()
+	out = p.communicate()[0].decode('UTF-8', 'replace').splitlines()
 	job.comments = {}
 	script = False
 	job.command = ''
@@ -166,7 +212,7 @@ def _parseScript(job):
 			except ValueError:
 				continue
 			try:
-				key, value = key.decode('unicode_escape'), value.decode('unicode_escape')
+				key, value = key.encode('UTF-8').decode('unicode_escape'), value.encode('UTF-8').decode('unicode_escape')
 			except UnicodeDecodeError:
 				pass  # can only happen if user manipulates/fakes atjob.
 			job.comments[key] = value
@@ -175,7 +221,14 @@ def _parseScript(job):
 
 
 def _parseJob(string):
-	'''Internal method to parse output of at-command.'''
+	# type: (str) -> Optional[AtJob]
+	"""
+	Internal method to parse output of :command:`atq`.
+
+	:param str string: A output line of :command:`atq`.
+	:returns: A :py:class:`AtJob` instance or `None`
+	:rtype: AtJob
+	"""
 	timeLocale = locale.getlocale(locale.LC_TIME)
 	try:
 		# change the time locale temporarily to 'C' as atq uses English date format
@@ -184,7 +237,7 @@ def _parseJob(string):
 
 		# parse string
 		tmp = _regWhiteSpace.split(string)
-		execTime = datetime.datetime.strptime(' '.join(tmp[1: 6]), _dateTimeFormatRead)
+		execTime = datetime.datetime.strptime(' '.join(tmp[1:6]), _dateTimeFormatRead)
 		isRunning = tmp[6] == '='
 		owner = tmp[7]
 		nr = int(tmp[0])
@@ -199,28 +252,40 @@ def _parseJob(string):
 
 
 class AtJob(object):
+	"""
+	This class is an abstract representation of an at-job. Do not initiate
+	the class directly, but use the methods provided in this module.
 
-	'''This class is an abstract representation of an at-job. Do not initiate
-	the class directly, but use the methods provided in this module.'''
-
+	:param int nr: Job number.
+	:param str owner: User owning the job.
+	:param datetime.datetime execTime: Planned job execution time.
+	:param bool isRunning: `True` is the jub is currently running, `False` otherwise.
+	"""
 	def __init__(self, nr, owner, execTime, isRunning):
+		# type: (int, str, datetime.datetime, bool) -> None
 		self.nr = nr
 		self.owner = owner
-		self.command = None
+		self.command = None  # type: Optional[str]
 		self.execTime = execTime
 		self.isRunning = isRunning
-		self.comments = {}
+		self.comments = {}  # type: Dict[str, str]
 
 	def __str__(self):
+		# type: () -> str
 		t = self.execTime.strftime(_dateTimeFormatWrite)
 		if self.isRunning:
 			t = 'running'
 		return 'Job #%d (%s)' % (self.nr, t)
 
 	def __repr__(self):
+		# type: () -> str
 		return self.__str__()
 
 	def rm(self):
-		'''Remove the job from the queue.'''
+		# type: () -> int
+		"""
+		Remove the job from the queue.
+		"""
 		p = subprocess.Popen(['/usr/bin/atrm', str(self.nr)], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 		p.communicate()
+		return p.returncode == 0
