@@ -325,6 +325,14 @@ class UniventionLDAPExtension(six.with_metaclass(ABCMeta)):
 				language = filename_parts[0]
 				with open(udm_module_messagecatalog, 'rb') as f:
 					common_udm_options.extend(["--append", "messagecatalog=%s %s" % (language, base64.b64encode(f.read()).decode('ASCII'),), ])
+
+			for umcmessagecatalog in options.umcmessagecatalog:
+				filename_parts = os.path.splitext(os.path.basename(umcmessagecatalog))
+				if not ('-' in filename_parts[0] and len(filename_parts[0].split('-', 1)) == 2):
+					raise OptionValueError("%s: Is not a valid umcmessagecatalog filename. Must be the locale and the UMCModuleID seperated by '-'" % (filename_parts[0],))
+				with open(umcmessagecatalog, 'rb') as f:
+					common_udm_options.extend(["--append", "umcmessagecatalog=%s %s" % (filename_parts[0], base64.b64encode(f.read()).decode('ASCII'),), ])
+
 			if options.umcregistration:
 				try:
 					with open(options.umcregistration, 'rb') as f:
@@ -1332,6 +1340,11 @@ def ucs_registerLDAPExtension():
 		type="existing_filename", default=[],
 		action="callback", callback=option_callback_append_udm_module_options,
 		help="Gettext mo file", metavar="<GNU message catalog file>")
+	udm_module_options.add_option(
+		"--umcmessagecatalog", dest="umcmessagecatalog",
+		type="existing_filename", default=[],
+		action="callback", callback=option_callback_append_udm_module_options,
+		help="Gettext mo file", metavar="<GNU message catalog file>")
 	udm_module_options.add_option(
 		"--umcregistration", dest="umcregistration",
 		type="existing_filename",
