@@ -1,11 +1,11 @@
-#!/usr/bin/python2.7
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
+"""
+Univention common Python library for :command:`logrotate` configuration files.
+"""
+# Copyright 2010-2021 Univention GmbH
 #
-# Univention Common Python Library
-#
-# Copyright 2010-2017 Univention GmbH
-#
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -28,11 +28,23 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
+
+from typing import Dict  # noqa F401
+
+from univention.config_registry import ConfigRegistry  # noqa F401
 
 
 def _getBoolDefault(varGlobal, varLocal, settings, configRegistry):
+	# type: (str, str, Dict[str, str], ConfigRegistry) -> None
+	"""
+	Get default value of type boolean.
 
+	:param str varGlobal: The |UCR| variable name of the global setting.
+	:param str varLocal: The |UCR| variable name of the service specific setting.
+	:param dict settings: A mapping, where the configuration is stored in.
+	:param ConfigRegistry configRegistry: An |UCR| instance.
+	"""
 	configName = varGlobal.split("/")[-1]
 	if configRegistry.is_true(varGlobal, True):
 		settings[configName] = configName
@@ -44,7 +56,19 @@ def _getBoolDefault(varGlobal, varLocal, settings, configRegistry):
 
 
 def getLogrotateConfig(name, configRegistry):
+	# type: (str, ConfigRegistry) -> Dict[str, str]
+	"""
+	Build aggregated configuration for log file rotation.
 
+	:param str name: The name of the log file or service.
+	:param ConfigRegistry configRegistry: An |UCR| instance.
+	:returns: A dictionary containing the merged configuration.
+	:rtype: dict
+
+	>>> ucr = ConfigRegistry()
+	>>> ucr.load()
+	>>> conf = getLogrotateConfig('service', ucr)
+	"""
 	settings = {}
 
 	for var in ["logrotate/", "logrotate/" + name + "/"]:
